#!/usr/local/bin/perl -w

# This provides a class for manipulating NxM matrices

package TiedMatrix;

use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

# Allow carping of errors
use Carp;

use Matrix;

# Can overload with the overload package by uncommenting the following
#  and filling in the appropriate functions
# use overload ('+' => \&matrix_add);

use Exporter;
$VERSION = 1.00;
@ISA = qw(Matrix);

@EXPORT      = qw();
@EXPORT_OK   = qw();
%EXPORT_TAGS = ();


sub new {
    my $class = shift;
    
    # Set up memory for the object and bless it into the class
    my $self = {};
    bless($self, $class);

    # Initialize the object with the remaining arguments
    $self->_init(@_);
    
    # And give the object back to the caller
    return $self;
}

sub TIEARRAY {
    my $class = shift;
    my $matrix = shift;
    my $self = {};
    bless($self, $class);

    print STDERR "TIEARRAY: $class\n";
    
    $self->{"MATRIX"} = $matrix;
    $self->{"M"} = undef;

    return $self;
}


sub FETCH {
    my $self = shift;
    my $index = shift;

    print STDERR "FETCH: $index\n";

    # Do we have all of the information we need?
    # If not then store this index and return ourself
    if (not defined $self->{"M"}) {
	$self->{"M"} = $index;
	return $self->dat;
    }    

    # We have all of the information
    my $m = $self->{"M"};
    $self->{"M"} = undef;

    print "M: $m\tN: $index\n";    
    my $ret = $self->{"MATRIX"}->data($m, $index);
    print "RET: $ret\n";
    return $ret;
}

sub STORE {
    print STDERR "STORE\n";
    FETCH(@_);
}


sub DESTORY {
    print STDERR "DESTROY\n";
}


sub dat {
    my $self = shift;

    return $self->{TIED};
}


sub _init {
    my $self = shift;

    $self->SUPER::_init(@_);
    
    # Now make the special tied element to allow funky access
    my @tied;
    tie @tied, "TiedMatrix", $self;
    $self->{TIED} = \@tied;

    return $self;
}

1;
