#!/usr/local/bin/perl -w

# This provides some modules that set up the UI

package UI;

use strict;

use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

# Allow carping of errors
use Carp;

# Use the TK routines
use Tk;
use Tk::LabEntry;

use Exporter;
$VERSION = 1.00;
@ISA = qw(Exporter);

@EXPORT      = qw(setup_ui);
@EXPORT_OK   = qw();
%EXPORT_TAGS = ();


sub setup_ui {
    my ($width, $height, $pos, $rot) = @_;

    # Start our window and init GL
    my $mw = MainWindow->new();
    my $gl = $mw->Canvas(-width => $width, -height => $height, 
			 -takefocus => 1)
	->pack(-side=>"top", -expand=>1, -fill => 'both');
    $gl->CanvasFocus;                # Take the initial focus

    # Add an exit button
    my $exit_b = $mw->Button(-text => 'Exit',
			     -command => sub { exit })
	->pack(-side=>"bottom", -expand=>0);
    
    # Add a rotate button
    my $spin_b = $mw->Button(-text => 'Start Spin')
	->pack(-side=>"bottom", -expand=>0);

    # Allow manipulation of the location
    add_framed_entries($mw, 'Position ', 'X: ', \$pos->{'x'}, 
                       'Y: ', \$pos->{'y'}, 'Z: ', \$pos->{'z'});
    
    # Allow manipulation of the rotation
    add_framed_entries($mw, 'Rotation ', 'X: ', \$rot->{'x'}, 
                       'Y: ', \$rot->{'y'}, 'Z: ', \$rot->{'z'});

    # Make a second window
    if (0) {
	my $tl = $mw->Toplevel();
	my $tree = $tl->Tree()->pack(-expand=>1, -fill=>'both', -side=>'left');
	
	foreach (qw(1 1.1 1.2 1.3 2 2.1 2.1.1 2.1.2 3)) {
	    $tree->add($_, -text=>$_);
	}

	$tree->autosetmode;
    }

    return ($mw, $gl, $spin_b);
}


sub add_framed_entries {
    my $mw = shift;
    my $label = shift;

    my $f = $mw->Frame()->pack(-side=>"top", -expand=>0);
    $f->Label(-text => $label)->pack(-side=>"left", -expand=>1);

    while ($label = shift) {
	my $variable = shift;
	$f->LabEntry(-label => $label, -width => 5,
		     -labelPack => [-side=>"left", -expand=>1],
		     -textvariable => $variable)
	    ->pack(-side=>"left", -expand=>0);
    }

    return $f;
}

1;
